// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DL_FRAME_UNARY_OPERATION_H
#define DL_FRAME_UNARY_OPERATION_H

#include "Node.h"
#include "FrameUnaryOperation.h"
#include "DLManager.h"

/**A FrameUnaryOperation that takes functions directly from a shared library*/
class DLFrameUnaryOperation : public FrameUnaryOperation {
public:

   /**Constructor, takes the name of the node and a set of parameters*/
   DLFrameUnaryOperation(string nodeName, ParameterSet params);

   /**Destructor*/
   ~DLFrameUnaryOperation();

   /**Class specific initialization routine.
      Each class will call its subclass specificInitialize() method*/
   virtual void specificInitialize();

   /**Class reset routine.
      Each class will call its superclass reset() method*/
   virtual void reset();

protected:
   /**if true, the we are calling a certain function for each element*/
   bool elementWise;

   /**Pointer to a loaded shared library (dlopen, shl_open, ...)*/
   LoadedLibrary *library;

   /**A pointer to a generic function that takes frame(s) as input and output*/
   void *computeFunct;

   /**internal data used by the called library*/
   void *internalData;

   /**Default constructor, should not be used*/
   DLFrameUnaryOperation() 
   {throw new GeneralException("DLFrameUnaryOperation copy constructor should not be called",__FILE__,__LINE__);}

   /**Computes the output for one frame*/
   virtual void computeFrame(ObjectRef input, int count);

};

#endif
