// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stream.h>
#include "FrameOperation.h"
#include "Buffer.h"
#include "Vector.h"
#include <values.h>
#include "misc.h"

class Max;

//DECLARE_NODE(Max)
NODE_INFO(Max, "Signal:Base", "INPUT", "OUTPUT", "INPUTLENGTH:OUTPUTLENGTH:START:END")

class Max : public FrameOperation {
   
   int inputID;
   int inputLength;
   int start;
   int end;

public:
   Max(string nodeName, ParameterSet params)
   : FrameOperation(nodeName, params)
   {
      inputID = addInput("INPUT");
      if (parameters.exist("INPUTLENGTH"))
         inputLength = dereference_cast<int> (parameters.get("INPUTLENGTH"));
      else inputLength = dereference_cast<int> (parameters.get("LENGTH"));
      start = dereference_cast<int> (parameters.get("START"));
      end = dereference_cast<int> (parameters.get("END"));

   }

   ~Max() {}

   virtual void specificInitialize()
   {
      this->FrameOperation::specificInitialize();
   }

   void calculate(int output_id, int count, Buffer &out)
   {
      int i;
      NodeInput input = inputs[inputID];
      ObjectRef inputValue = input.node->getOutput(input.outputID, count);

      Vector<float> &output = object_cast<Vector<float> > (out[count]);
      if (inputValue->status != Object::valid)
      {
         output.status = inputValue->status;
         return;
      }
      const Vector<float> &in = object_cast<Vector<float> > (inputValue);
      
      /*output[0]=in[0]*in[0];
      for (int i=1;i<outputLength;i++)
      {
         output[i]=in[i]*in[i]+in[inputLength-i]*in[inputLength-i];
         }*/

      
      int maxcep;
      float maxval;
      
      maxval=-FLT_MAX;
      maxcep=0;
      for (i=start;i<=end;i++)         /*find the maximum cep (positive only)*/
      {
         if (in[i]>maxval) 
         {
            maxcep=i;
            maxval=in[i];
         }
      }      
      
      output[0]=maxval;
      output[1]=maxcep;
      

      output.status = Object::valid;
   }

};
