// Copyright (C) 1999 Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/*
** Written by Andre Charbonneau
** June 14 1999
**
** Description:
** 
** History:
** - June 14 1999: Created.
*/

#include "FP_network_info_container.h"
#include "FileParserKeys.h"

FP_network_info_container::FP_network_info_container()
{
  m_net_name = "";
  m_input_node_name = "";
  m_output_node_name = "";
  m_condition_node_name = "";
  m_iterator = false;
  return;
}

FP_network_info_container::FP_network_info_container(const string& net_name, const bool iterator_net)
{
  m_net_name = net_name;
  m_input_node_name = "";
  m_output_node_name = "";
  m_iterator = iterator_net;
  m_condition_node_name = "";
  return;
}

FP_network_info_container::FP_network_info_container(const FP_network_info_container& netic)
{
  m_net_name = netic.m_net_name;
  m_input_node_name = netic.m_input_node_name;
  m_output_node_name = netic.m_output_node_name;
  m_nodes_map = netic.m_nodes_map;
  m_iterator = netic.m_iterator;
  m_condition_node_name = netic.m_condition_node_name;
  return;
}

FP_network_info_container::~FP_network_info_container()
{
  // nothing to do...
  return;
}

FP_network_info_container& FP_network_info_container::operator=(const FP_network_info_container& nic)
{
  m_net_name = nic.m_net_name;
  m_nodes_map = nic.m_nodes_map;
  m_input_node_name = nic.m_input_node_name;
  m_output_node_name = nic.m_output_node_name;
  m_iterator = nic.m_iterator;
  m_condition_node_name = nic.m_condition_node_name;

  return *this;
}

string FP_network_info_container::get_name() const
{
  return m_net_name;
}

void FP_network_info_container::set_name(const string& s)
{
  m_net_name = s;
  return;
}

void FP_network_info_container::set_input_node_name(const string& input_node_name)
{
  m_input_node_name = input_node_name;
  return;
}

string FP_network_info_container::get_input_node_name() const
{
  return m_input_node_name;
}

void FP_network_info_container::set_output_node_name(const string& output_node_name)
{
  m_output_node_name = output_node_name;
  return;
}

string FP_network_info_container::get_output_node_name() const
{
  return m_output_node_name;
}

void FP_network_info_container::set_condition_node_name(const string& condition_node_name)
{
  m_condition_node_name = condition_node_name;
  return;
}

string FP_network_info_container::get_condition_node_name() const
{
  return m_condition_node_name;
}


void FP_network_info_container::add_node(const string& node_name) throw(FileParserException)
{
  FP_node_info_container nodeic;
  nodeic.set_node_name(node_name);
  if(m_nodes_map.count(node_name) > 0)
    {
      // duplicate error
      throw(FileParserException(DUPLICATE_EXCEPTION,string("Tried to add node ") + node_name + string(" twice to a network.")));
    }

  m_nodes_map[node_name] = nodeic;

  return;
}

void FP_network_info_container::add_input_to_node(const string& node_name, const input_var_info& ii) throw(FileParserException)
{
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception...
      throw(FileParserException(NOT_FOUND_EXCEPTION, string("Node: ") + node_name + string(" not found.")));
    }
  try
    {
      m_nodes_map[node_name].add_input_var(ii.var_name, ii.source_node_name, ii.source_var_name);
    }
  catch(FileParserException)
    {
      throw;
    }

  return;
}

input_var_info FP_network_info_container::get_input_from_node(const string& node_name, const string& input_name) const throw(FileParserException)
{
  input_var_info return_value;
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception...
      throw(FileParserException(NOT_FOUND_EXCEPTION, string("Node: ") + node_name + string(" not found.")));
    }

  try
    {
      return_value = m_nodes_map.find(node_name)->second.get_input_var_info(input_name);
    }
  catch(FileParserException)
    {
      throw;
    }

  return return_value;
}

void FP_network_info_container::add_param_to_node(const string& node_name, const param_info& pi) throw(FileParserException)
{
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception...
      throw(FileParserException(NOT_FOUND_EXCEPTION, string("Node: ") + node_name + string(" not found.")));
    }
  try
    {
      m_nodes_map[node_name].add_param_var(pi.param_name, pi.param_value, pi.param_type);
    }
  catch(FileParserException)
    {
      throw;
    }

  return;
}

void FP_network_info_container::modify_param_to_node(const string& node_name, const param_info& pi) throw(FileParserException)
{  
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception...
      throw(FileParserException(NOT_FOUND_EXCEPTION, string("Node: ") + node_name + string(" not found.")));
    }
  try
    {
      m_nodes_map[node_name].modify_param_var(pi.param_name, pi.param_value, pi.param_type);
    }
  catch(FileParserException)
    {
      throw;
    }

  return;
}

param_info FP_network_info_container::get_param_from_node(const string& node_name, const string& param_name) const throw(FileParserException)
{
  param_info return_value;
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception...
      throw(FileParserException(NOT_FOUND_EXCEPTION, string("Node: ") + node_name + string(" not found.")));
    }

  try
    {
      return_value = m_nodes_map.find(node_name)->second.get_param_info(param_name);
    }
  catch(FileParserException)
    {
      throw;
    }

  return return_value;
}

list<param_info> FP_network_info_container::get_params_from_node(const string& node_name) const throw(FileParserException)
{
  list<param_info> pi_list;

  try
    {
      list<string> param_names = get_param_var_names_from_node(node_name);
      for(list<string>::const_iterator i = param_names.begin(); !(i == param_names.end()); i++)
	{
	  pi_list.push_back(get_param_from_node(node_name,*i));
	}
    }
  catch(FileParserException)
    {
      throw;
    }
  
  return pi_list;
}

string FP_network_info_container::get_type_of_node(const string& node_name) const throw(FileParserException)
{
  string return_value = "";
  map<string,FP_node_info_container>::const_iterator iter = m_nodes_map.find(node_name);
  if(iter == m_nodes_map.end())
    {
      throw(FileParserException(NOT_FOUND_EXCEPTION,string("Node: ") + node_name + string(" not found.")));
    }
  else
    {
      return_value = iter->second.get_node_type();
    }

  return return_value;
}

void FP_network_info_container::set_type_of_node(const string& node_name, const string& node_type) throw(FileParserException)
{
  map<string,FP_node_info_container>::iterator iter = m_nodes_map.find(node_name);
  if(iter == m_nodes_map.end())
    {
      throw(FileParserException(NOT_FOUND_EXCEPTION,string("Node: ") + node_name + string(" not found.")));
    }
  else
    {
      iter->second.set_node_type(node_type);
    }

  return;
}


list<string> FP_network_info_container::get_node_names() const
{
  list<string> return_value;

  for(map<string,FP_node_info_container>::const_iterator i = m_nodes_map.begin(); i != m_nodes_map.end(); i++)
    {
      return_value.push_back(i->first);
    }

  return return_value;
}

list<string> FP_network_info_container::get_input_var_names_from_node(const string& node_name) const throw(FileParserException)
{
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception.
      throw(FileParserException(NOT_FOUND_EXCEPTION,string("Node: ") + node_name + string(" not found.")));
    }

  return m_nodes_map.find(node_name)->second.get_input_var_names();
}

list<string> FP_network_info_container::get_param_var_names_from_node(const string& node_name) const throw(FileParserException)
{
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception.
      throw(FileParserException(NOT_FOUND_EXCEPTION,string("Node: ") + node_name + string(" not found.")));
    }

  return m_nodes_map.find(node_name)->second.get_param_var_names();
}

void FP_network_info_container::reset_params_from_node(const string& node_name) throw(FileParserException)
{
  if(m_nodes_map.count(node_name) == 0)
    {
      // node not found, throw an exception.
      throw(FileParserException(NOT_FOUND_EXCEPTION,string("Node: ") + node_name + string(" not found.")));
    }

  m_nodes_map.find(node_name)->second.reset_params();
  return;
}


bool FP_network_info_container::is_iterator() const
{
  return m_iterator;
}


ostream& operator<<(ostream& os, const FP_network_info_container& nic)
{
  if(nic.is_iterator())
    {
      os << ITERATOR_NET_BLOCK_KEY << ": " << nic.get_name() << endl
	 << NETWORK_START_DELIMETER << endl;
    }
  else
    {
      os << NET_BLOCK_KEY << ": " << nic.get_name() << endl
	 << NETWORK_START_DELIMETER << endl;
      
    }
    
  for(map<string,FP_node_info_container>::const_iterator i = nic.m_nodes_map.begin();
      i != nic.m_nodes_map.end();
      i++)
    {
      os << "\t" << i->second << endl << endl;
    }

  if(nic.get_input_node_name() != "")
    {
      os << "\t" << "<" << NET_INPUT_KEY <<": " << nic.get_input_node_name() << ">" << endl;
    }
  if(nic.get_output_node_name() != "")
    {
      os << "\t" << "<" << NET_OUTPUT_KEY << ": " << nic.get_output_node_name() << ">" << endl;
    }
  if(nic.get_condition_node_name() != "")
    {
      os << "\t" << "<" << NET_CONDITION_KEY << ": " << nic.get_condition_node_name() << ">" << endl;
    }

  os << NETWORK_END_DELIMETER;


  return os;
}
