#include "UINodeParameters.h"
#include <tree.h>
#include "Node.h"
#include "UINode.h"
#include "UINetwork.h"
#include "UIDocument.h"

#include "ParameterSet.h"

class ParamTypeChange {
public:
   char *newParam;
   string &value;
   
};

const vector<string> &UINodeParameters::allTypes()
{
   static vector<string> types;
   static int init=false;
   if (!init)
   {
      types.insert(types.end(), "int");
      types.insert(types.end(), "float");
      types.insert(types.end(), "string");
      types.insert(types.end(), "bool");
      types.insert(types.end(), "subnet_param");
      init=true;
   }
   return types;
}



UINodeParameters::UINodeParameters(UINode *_node, string type)
   : node(_node)
{
   //cerr << "UINodeParameters::UINodeParameters\n";
   int i;
   _NodeFactory *factory = Node::getFactoryNamed(type);
   if (factory)
   {
      vector<string> tmp=factory->getParams();
      params.resize(tmp.size());
      for (i=0;i<tmp.size();i++)
	 params[i].name=tmp[i];
   } else {
      vector<string> tmp = node->getNetwork()->getDocument()->getNetParams(type);
      params.resize(tmp.size());
      for (i=0;i<tmp.size();i++)
	 params[i].name=tmp[i];
      //cerr << "Factory not found in simple nodes... must be a subnet...\n";
   }

}

UINodeParameters::~UINodeParameters()
{
}

void UINodeParameters::insertLoadedParam(ParameterData *param, string type, string value)
{
}

void UINodeParameters::load(xmlNodePtr node)
{
   //cerr << "node = " << node << endl;
   xmlNodePtr par = node->childs;
   //cerr << "par = " << par << endl;
   while (par)
   {
      if (string((char*)par->name) == "Parameter")
      {
         string name = string ((char *) xmlGetProp(par, (CHAR *)"name"));
         string type = string ((char *) xmlGetProp(par, (CHAR *)"type"));
         string value = string ((char *) xmlGetProp(par, (CHAR *)"value"));
         
         ParameterData *param = getParamNamed(name);
	 if (param)
	 {
	    param->type = type;
	    param->value = value;
	    insertLoadedParam(param, type, value);
	    //cerr << "<param: " << name << ", " << type << ":" << value << ">\n";
	 } else {
	    cerr << "param " << name << " no longer used\n";
	 }
      }
      par = par->next;
      
   }
}



void UINodeParameters::saveXML(xmlNode *root)
{
   for (int i=0;i<params.size();i++)
   {
      if (params[i].value != "")
      {
         xmlNodePtr tree = xmlNewChild(root, NULL, (CHAR *)"Parameter", NULL);
         xmlSetProp(tree, (CHAR *)"name", (CHAR *)params[i].name.c_str());
         xmlSetProp(tree, (CHAR *)"type", (CHAR *)params[i].type.c_str());
         xmlSetProp(tree, (CHAR *)"value", (CHAR *)params[i].value.c_str());
      }
   }
   
}

void UINodeParameters::export2net(ostream &out)
{
   for (int i=0;i<params.size();i++)
   {
      if (params[i].value != "")
      {
	 if (params[i].type != "string")
	    out << "   <param: " << params[i].name << ", " 
		<< params[i].type << ":" << params[i].value << ">" << endl;
	 else 
	    out << "   <param: " << params[i].name << ", " 
		<< params[i].type << ":\"" << params[i].value << "\">" << endl;
      }
   }
}

ParameterData *UINodeParameters::getParamNamed(string n)
{
   for (int i=0;i<params.size();i++)
      if (params[i].name == n)
         return &(params[i]);
   return NULL;
}

void UINodeParameters::insertNetParams(vector<string> &par)
{
   for (int i=0;i<params.size();i++)
   {
      if (params[i].value != "" && params[i].type == "subnet_param")
      {
	 bool alreadyPresent = false;
	 for (int j=0;j<par.size();j++)
	    if (par[j] == params[i].value)
	       alreadyPresent=true;
	 if (!alreadyPresent)
	    par.insert(par.end(), params[i].value);
      }
   }
}

ParameterSet *UINodeParameters::build(const ParameterSet &par)
{
   ParameterSet *parameters = new ParameterSet;

   for (int i=0;i<params.size();i++)
   {
      ParameterData &curr = params[i];
      if (curr.value == "")
	 continue;
      ObjectRef value;
      if (curr.type == "int")
      {
	 int val = atoi (curr.value.c_str());
	 value = ObjectRef(new Int(val));
      } 
      else if (curr.type == "bool")
      {
	 if (curr.value == "true")
	    value = ObjectRef(new Bool(true));
	 else 
	    value = ObjectRef(new Bool(false));
      } 
      else if (curr.type == "float")
      {
	 float val = atof (curr.value.c_str());
	 value = ObjectRef(new Float(val)); 
      } 
      else if (curr.type == "string")
      {
	 value = ObjectRef(new String(curr.value));	 	 
      } 
      else if (curr.type == "subnet_param")
      {
	 //BUG WARNING
	 //I guess I shouldn't do that...
	 value = const_cast<ParameterSet &> (par).get(curr.value);
	 //cerr << "Subnet_params not supported\n"; 	 	 
      } 
      else {
	 cerr << "UNKNOWN PARAM TYPE: \"" << curr.type << "\"" << endl;
      }
      
      parameters->add(curr.name,value);
   }
   return parameters;
   //perform suvstitution
}
